@@ -10,6 +10,26 @@ The Fourier transforms are defined in the following way:
 Definitions
 -----------
 
+real time → complex frequencies
+^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+The Laplace integral for the Green's function is defined as
+
+.. math:: G(z) = ∫_{-∞}^{∞} dt G(t) \exp(izt)
+
+This integral is only well defined
+
+* in the upper complex half-plane `z.imag>=0` for retarded Green's function :math:`∝θ(t)`
+* in the lower complex half-plane `z.imag<=0` for advanced Green's function :math:`∝θ(-t)`
+
+The recommended high-level function to perform this Laplace transform is:
+
+* `tt2z` for both retarded and advanced Green's function
+
+Currently, to sub-functions can be used equivalently, the abstraction `tt2z` is
+mostly for consistency with the imaginary time ↔ Matsubara frequencies
+Fourier transformations.
+
 imaginary time → Matsubara frequencies
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
@@ -60,12 +80,32 @@ import logging
 import numpy as np
 from numpy import newaxis
 
-import gftool as gt
-from gftool.basis.pole import PoleGf
+from gftool._util import _gu_matvec
+from gftool.statistics import matsubara_frequencies, matsubara_frequencies_b
+from gftool.basis.pole import PoleFct, PoleGf
+
+try:
+    import numexpr as ne
+except ImportError:
+    _HAS_NUMEXPR = False
+else:
+    _HAS_NUMEXPR = True
+
 
 LOGGER = logging.getLogger(__name__)
 
 
+def _phase_numexpr(z, tt):
+    return ne.evaluate('exp(1j*z*tt)', local_dict={'z': z, 'tt': tt})
+
+
+def _phase_numpy(z, tt):
+    return np.exp(1j*z*tt)
+
+
+_phase = _phase_numexpr if _HAS_NUMEXPR else _phase_numpy
+
+
 def iw2tau_dft(gf_iw, beta):
     r"""Discrete Fourier transform of the Hermitian Green's function `gf_iw`.
 
@@ -102,7 +142,6 @@ def iw2tau_dft(gf_iw, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> iws = gt.matsubara_frequencies(range(1024), beta=BETA)
     >>> tau = np.linspace(0, BETA, num=2*iws.size + 1, endpoint=True)
@@ -187,7 +226,6 @@ def iw2tau_dft_soft(gf_iw, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> iws = gt.matsubara_frequencies(range(1024), beta=BETA)
     >>> tau = np.linspace(0, BETA, num=2*iws.size + 1, endpoint=True)
@@ -248,12 +286,12 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft, n_fit=0):
 
     Parameters
     ----------
-    gf_iw : (N_iw) complex np.ndarray
+    gf_iw : (..., N_iw) complex np.ndarray
         The Green's function at positive **fermionic** Matsubara frequencies
         :math:`iω_n`.
     beta : float
         The inverse temperature :math:`beta = 1/k_B T`.
-    moments : (m) float array_like
+    moments : (..., m) float array_like
         High-frequency moments of `gf_iw`.
     fourier : {`iw2tau_dft`, `iw2tau_dft_soft`}, optional
         Back-end to perform the actual Fourier transformation.
@@ -263,7 +301,7 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft, n_fit=0):
 
     Returns
     -------
-    gf_tau : (2*N_iw + 1) float np.ndarray
+    gf_tau : (..., 2*N_iw + 1) float np.ndarray
         The Fourier transform of `gf_iw` for imaginary times :math:`τ \in [0, β]`.
 
     See Also
@@ -282,7 +320,6 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft, n_fit=0):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> iws = gt.matsubara_frequencies(range(1024), beta=BETA)
     >>> tau = np.linspace(0, BETA, num=2*iws.size + 1, endpoint=True)
@@ -334,7 +371,7 @@ def iw2tau(gf_iw, beta, moments=(1.,), fourier=iw2tau_dft, n_fit=0):
 
     """
     moments = np.asarray(moments)
-    iws = gt.matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
+    iws = matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
     # newaxis in pole_gf inserts axis for iws/tau
     if n_fit:
         n_mom = moments.shape[-1]
@@ -369,7 +406,7 @@ def tau2iv_dft(gf_tau, beta):
 
     Returns
     -------
-    gf_iv : (..., {N_iv + 1}/2) float np.ndarray
+    gf_iv : (..., (N_iv + 1)/2) float np.ndarray
         The Fourier transform of `gf_tau` for non-negative bosonic Matsubara
         frequencies :math:`iν_n`.
 
@@ -379,7 +416,6 @@ def tau2iv_dft(gf_tau, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> ivs = gt.matsubara_frequencies_b(range((tau.size+1)//2), beta=BETA)
@@ -451,7 +487,7 @@ def tau2iv_ft_lin(gf_tau, beta):
 
     Returns
     -------
-    gf_iv : (..., {N_iv + 1}/2) float np.ndarray
+    gf_iv : (..., (N_iv + 1)/2) float np.ndarray
         The Fourier transform of `gf_tau` for non-negative bosonic Matsubara
         frequencies :math:`iν_n`.
 
@@ -469,7 +505,6 @@ def tau2iv_ft_lin(gf_tau, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> ivs = gt.matsubara_frequencies_b(range((tau.size+1)//2), beta=BETA)
@@ -515,11 +550,11 @@ def tau2iv_ft_lin(gf_tau, beta):
     gf_dft = np.fft.ihfft(gf_tau[..., :-1])
     d_gf_dft = np.fft.ihfft(gf_tau[..., 1:] - gf_tau[..., :-1])
     d_tau_ivs = 2j*np.pi/(n_tau - 1)*np.arange(gf_dft.shape[-1])
-    d_tau_ivs[..., 0] = np.nan  # avoid zero division, fix value by hand
+    d_tau_ivs[..., 0] = 1  # avoid zero division, fix value by hand later
     expm1 = np.expm1(d_tau_ivs)
     weight1 = expm1/d_tau_ivs
     weight2 = (expm1 + 1 - weight1)/d_tau_ivs
-    weight1[0], weight2[0] = 1, .5  # special case n=0
+    weight1[..., 0], weight2[..., 0] = 1, .5  # special case n=0, fix from before
     gf_iv = weight1*gf_dft + weight2*d_gf_dft
     gf_iv = beta*gf_iv
     return gf_iv
@@ -549,7 +584,7 @@ def tau2iv(gf_tau, beta, fourier=tau2iv_ft_lin):
 
     Returns
     -------
-    gf_iv : (..., {N_iv + 1}/2) complex np.ndarray
+    gf_iv : (..., (N_iv + 1)/2) complex np.ndarray
         The Fourier transform of `gf_tau` for non-negative bosonic Matsubara
         frequencies :math:`iν_n`.
 
@@ -560,7 +595,6 @@ def tau2iv(gf_tau, beta, fourier=tau2iv_ft_lin):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> ivs = gt.matsubara_frequencies_b(range((tau.size+1)//2), beta=BETA)
@@ -610,7 +644,7 @@ def tau2iv(gf_tau, beta, fourier=tau2iv_ft_lin):
     tau = np.linspace(0, beta, num=gf_tau.shape[-1])
     gf_tau = gf_tau - g1[..., newaxis]/beta*tau  # remove jump by linear shift
     gf_iv = fourier(gf_tau, beta=beta)
-    ivs = gt.matsubara_frequencies_b(range(1, gf_iv.shape[-1]), beta=beta)
+    ivs = matsubara_frequencies_b(range(1, gf_iv.shape[-1]), beta=beta)
     gf_iv[..., 1:] += g1[..., newaxis]/ivs
     gf_iv[..., 0] += .5 * g1 * beta  # `iv_{n=0}` = 0 has to be treated separately
     return gf_iv
@@ -636,7 +670,7 @@ def tau2iw_dft(gf_tau, beta):
 
     Returns
     -------
-    gf_iw : (..., {N_iw - 1}/2) float np.ndarray
+    gf_iw : (..., (N_iw - 1)/2) float np.ndarray
         The Fourier transform of `gf_tau` for positive fermionic Matsubara
         frequencies :math:`iω_n`.
 
@@ -646,7 +680,6 @@ def tau2iw_dft(gf_tau, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> iws = gt.matsubara_frequencies(range((tau.size-1)//2), beta=BETA)
@@ -719,7 +752,7 @@ def tau2iw_ft_lin(gf_tau, beta):
 
     Returns
     -------
-    gf_iw : (..., {N_iw - 1}/2) float np.ndarray
+    gf_iw : (..., (N_iw - 1)/2) float np.ndarray
         The Fourier transform of `gf_tau` for positive fermionic Matsubara
         frequencies :math:`iω_n`.
 
@@ -737,7 +770,6 @@ def tau2iw_ft_lin(gf_tau, beta):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> iws = gt.matsubara_frequencies(range((tau.size-1)//2), beta=BETA)
@@ -823,7 +855,7 @@ def tau2iw(gf_tau, beta, n_pole=None, moments=None, fourier=tau2iw_ft_lin):
 
     Returns
     -------
-    gf_iw : (..., {N_iv + 1}/2) complex np.ndarray
+    gf_iw : (..., (N_iv + 1)/2) complex np.ndarray
         The Fourier transform of `gf_tau` for non-negative fermionic Matsubara
         frequencies :math:`iω_n`.
 
@@ -836,7 +868,6 @@ def tau2iw(gf_tau, beta, n_pole=None, moments=None, fourier=tau2iw_ft_lin):
 
     Examples
     --------
-    >>> import gftool.fourier
     >>> BETA = 50
     >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
     >>> iws = gt.matsubara_frequencies(range((tau.size-1)//2), beta=BETA)
@@ -944,6 +975,486 @@ def tau2iw(gf_tau, beta, n_pole=None, moments=None, fourier=tau2iw_ft_lin):
                               moments=moments[..., newaxis, :])
     gf_tau = gf_tau - pole_gf.eval_tau(tau, beta)
     gf_iw = fourier(gf_tau, beta=beta)
-    iws = gt.matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
+    iws = matsubara_frequencies(range(gf_iw.shape[-1]), beta=beta)
     gf_iw += pole_gf.eval_z(iws)
     return gf_iw
+
+
+def _z2polegf(z, gf_z, n_pole, moments=(1.,)) -> PoleFct:
+    moments = np.asanyarray(moments)
+
+    def error_(width):
+        pole_gf = PoleFct.from_z(z, gf_z, n_pole=n_pole,
+                                 # if width is 0, no higher moments exist
+                                 moments=moments if width else moments[..., 0:1], width=width)
+        gf_fit = pole_gf.eval_z(z)
+        return np.linalg.norm(gf_z - gf_fit)
+
+    from scipy.optimize import minimize_scalar  # pylint: disable=import-outside-toplevel
+    opt = minimize_scalar(error_)
+    LOGGER.debug("Fitting error: %s Optimal pole-spread: %s", opt.fun, opt.x)
+    opt_pole_gf = PoleFct.from_z(z, gf_z, n_pole=n_pole, moments=moments, width=opt.x)
+    return opt_pole_gf
+
+
+def izp2tau(izp, gf_izp, tau, beta, moments=(1.,)):
+    r"""Fourier transform of the Hermitian Green's function `gf_izp` to `tau`.
+
+    Fourier transformation of a fermionic Padé Green's function to
+    imaginary-time domain.
+    We assume a Hermitian Green's function `gf_iw`, i.e. :math:`G(-iω_n) = G^*(iω_n)`,
+    which is the case for commutator Green's functions :math:`G_{AB}(τ) = ⟨A(τ)B⟩`
+    with :math:`A = B^†`. The Fourier transform `gf_tau` is then real.
+
+    TODO: this function is not vectorized yet.
+
+    Parameters
+    ----------
+    izp, gf_iw : (N_izp) float np.ndarray
+        Positive **fermionic** Padé frequencies :math:`iz_p` and the Green's
+        function at specified frequencies.
+    tau : (N_tau) float np.ndarray
+        Imaginary times `0 <= tau <= beta` at which the Fourier transform is
+        evaluated.
+    beta : float
+        The inverse temperature :math:`beta = 1/k_B T`.
+    moments : (m) float array_like, optional
+        High-frequency moments of `gf_iw`.
+
+    Returns
+    -------
+    gf_tau : (N_tau) float np.ndarray
+        The Fourier transform of `gf_izp` for imaginary times `tau`.
+
+    See Also
+    --------
+    iw2tau : Fourier transform from fermionic Matsubara frequencies.
+    _z2polegf : Function handling the fitting of `gf_izp`
+
+    Notes
+    -----
+    The algorithm performs in fact an analytic continuation instead of a
+    Fourier integral. It is however only evaluated on the imaginary axis, so
+    far the algorithm was observed to be stable
+
+    Examples
+    --------
+    >>> BETA = 50
+    >>> izp, __ = gt.pade_frequencies(50, beta=BETA)
+    >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
+
+    >>> poles = 2*np.random.random(10) - 1  # partially filled
+    >>> weights = np.random.random(10)
+    >>> weights = weights/np.sum(weights)
+    >>> gf_izp = gt.pole_gf_z(izp, poles=poles, weights=weights)
+    >>> gf_ft = gt.fourier.izp2tau(izp, gf_izp, tau, beta=BETA)
+    >>> gf_tau = gt.pole_gf_tau(tau, poles=poles, weights=weights, beta=BETA)
+
+    >>> import matplotlib.pyplot as plt
+    >>> __ = plt.plot(tau, gf_tau, label='exact')
+    >>> __ = plt.plot(tau, gf_ft, '--', label='FT')
+    >>> __ = plt.legend()
+    >>> plt.show()
+
+    >>> __ = plt.title('Oscillations of tiny magnitude')
+    >>> __ = plt.plot(tau/BETA, gf_tau - gf_ft)
+    >>> __ = plt.xlabel('τ/β')
+    >>> plt.show()
+
+    Results of `izp2tau` can be improved giving high-frequency moments.
+
+    >>> mom = np.sum(weights[:, np.newaxis] * poles[:, np.newaxis]**range(4), axis=0)
+    >>> for n in range(1, 4):
+    ...     gf = gt.fourier.izp2tau(izp, gf_izp, tau, beta=BETA, moments=mom[:n])
+    ...     __ = plt.plot(tau, abs(gf_tau - gf), label=f'n_mom={n}')
+    >>> __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.show()
+
+    The method is resistant against noise:
+
+    >>> magnitude = 2e-7
+    >>> noise = np.random.normal(scale=magnitude, size=gf_izp.size)
+    >>> gf = gt.fourier.izp2tau(izp, gf_izp + noise, tau, beta=BETA, moments=(1,))
+    >>> __ = plt.plot(tau/BETA, abs(gf_tau - gf))
+    >>> __ = plt.axhline(magnitude, color='black')
+    >>> plt.yscale('log')
+    >>> plt.tight_layout()
+    >>> plt.show()
+
+    >>> for n in range(1, 4):
+    ...     gf = gt.fourier.izp2tau(izp, gf_izp + noise, tau, beta=BETA, moments=mom[:n])
+    ...     __ = plt.plot(tau/BETA, abs(gf_tau - gf), '--', label=f'n_mom={n}')
+    >>> __ = plt.axhline(magnitude, color='black')
+    >>> __ = plt.plot(tau/BETA, abs(gf_tau - gf_ft), label='clean')
+    >>> __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.tight_layout()
+    >>> plt.show()
+
+    """
+    pole_gf = PoleGf(*_z2polegf(izp, gf_izp, n_pole=izp.size, moments=moments))
+    return pole_gf.eval_tau(tau, beta)
+
+
+def tt2z_trapz(tt, gf_t, z):
+    r"""Laplace transform of the real-time Green's function `gf_t`.
+
+    Approximate the Laplace integral by trapezoidal rule:
+
+    .. math::
+
+       G(z) = ∫dt G(t) \exp(izt)
+            ≈ ∑_{k=1}^N [G(t_{k-1})\exp(izt_{k-1}) + G(t_k)\exp(izt_k)] Δt_k/2
+
+    The function can handle any input discretization `tt`.
+
+    Parameters
+    ----------
+    tt : (Nt) float np.ndarray
+        The points for which the Green's function `gf_t` is given.
+    gf_t : (..., Nt) complex np.ndarray
+        Green's function and time points `tt`.
+    z : (..., Nz) complex np.ndarray
+        Frequency points for which the Laplace transformed Green's function
+        should be evaluated.
+
+    Returns
+    -------
+    gf_z : (..., Nz) complex np.ndarray
+        Laplace transformed Green's function for complex frequencies `z`.
+
+    See Also
+    --------
+    tt2z_lin : Laplace integration using Filon's method
+
+    Notes
+    -----
+    The function is equivalent to the one-liner
+    `np.trapz(np.exp(1j*z[:, None]*tt)*gf_t, x=tt)`.
+    Internally this function evaluates the sum as a matrix product to leverage
+    the speed-up of BLAS. If `numexpr` is available, it is used for the speed
+    up it provides for transcendental equations.
+
+    """
+    phase = _phase(z[..., newaxis], tt[newaxis, :])
+    boundary = (phase[..., 0]*gf_t[..., :1]*(tt[1] - tt[0])
+                + phase[..., -1]*gf_t[..., -1:]*(tt[-1] - tt[-2]))
+    d2tt = tt[2:] - tt[:-2]
+    trapz = _gu_matvec(phase[..., 1:-1], gf_t[..., 1:-1]*d2tt)
+    return 0.5*(boundary + trapz)
+
+
+def tt2z_lin(tt, gf_t, z):
+    r"""Laplace transform of the real-time Green's function `gf_t`.
+
+    Filon's method is used to calculate the Laplace integral
+
+    .. math:: G(z) = ∫dt G(t) \exp(izt),
+
+    :math:`G(t)` is approximated by a linear spline.
+    The function currently requires an equidistant `tt`.
+    Information on oscillatory integrations can be found e.g. in [filon1928]_
+    and [iserles]_.
+
+    Parameters
+    ----------
+    tt : (Nt) float np.ndarray
+        The equidistant points for which the Green's function `gf_t` is given.
+    gf_t : (..., Nt) complex np.ndarray
+        Green's function and time points `tt`.
+    z : (..., Nz) complex np.ndarray
+        Frequency points for which the Laplace transformed Green's function
+        should be evaluated.
+
+    Returns
+    -------
+    gf_z : (..., Nz) complex np.ndarray
+        Laplace transformed Green's function for complex frequencies `z`.
+
+    Raises
+    ------
+    ValueError
+        If the time points `tt` are not equidistant.
+
+    See Also
+    --------
+    tt2z_trapz : Plain implementation using trapezoidal rule.
+
+    Notes
+    -----
+    Internally this function evaluates the sum as a matrix product to leverage
+    the speed-up of BLAS. If `numexpr` is available, it is used for the speed
+    up it provides for transcendental equations.
+
+    References
+    ----------
+    .. [filon1928] L.N. Filon, On a quadrature formula for trigonometric integrals,
+       Proc. Roy. Soc. Edinburgh 49 (1928) 38-47.
+    .. [iserles] A. Iserles, S.P. Nørsett, and S. Olver, Highly oscillatory
+       quadrature: The story so far,
+       http://www.sam.math.ethz.ch/~hiptmair/Seminars/OSCINT/INO06.pdf
+
+    """
+    delta_tt = tt[1] - tt[0]
+    if not np.allclose(tt[1:] - tt[:-1], delta_tt):
+        raise ValueError("Equidistant `tt` required for current implementation.")
+    zero = z == 0  # special case `z=0` has to be handled separately (due: 1/z)
+    if np.any(zero):
+        z = np.where(zero, 1, z)
+    izdt = 1j*z*delta_tt
+    phase = _phase(z[..., newaxis], tt[newaxis, :-1])
+    g_dft = _gu_matvec(phase, gf_t[..., :-1])
+    dg_dft = _gu_matvec(phase, gf_t[..., 1:] - gf_t[..., :-1])
+    weight1 = np.expm1(izdt)/izdt
+    weight2 = (np.exp(izdt) - weight1)/izdt
+    gf_z = delta_tt * (weight1*g_dft + weight2*dg_dft)
+    if np.any(zero):
+        gf_z[..., zero] = np.trapz(gf_t, x=tt)[..., np.newaxis]
+    return gf_z
+
+
+def tt2z(tt, gf_t, z, laplace=tt2z_lin):
+    r"""Laplace transform of the real-time Green's function `gf_t`.
+
+    Calculate the Laplace transform
+
+    .. math:: G(z) = ∫dt G(t) \exp(izt)
+
+    For the Laplace transform to be well defined,
+    it should either be `tt>=0 and z.imag>=0` for the retarded Green's function,
+    or `tt<=0 and z.imag<=0` for the advance Green's function.
+
+    The retarded (advanced) Green's function can in principle be evaluated for
+    any frequency point `z` in the upper (lower) complex half-plane.
+
+    The accented contours for `tt` and `z` depend on the specific used back-end
+    `laplace`.
+
+    Parameters
+    ----------
+    tt : (Nt) float np.ndarray
+        The points for which the Green's function `gf_t` is given.
+    gf_t : (..., Nt) complex np.ndarray
+        Green's function and time points `tt`.
+    z : (..., Nz) complex np.ndarray
+        Frequency points for which the Laplace transformed Green's function
+        should be evaluated.
+    laplace : {`tt2z_lin`, `tt2z_trapz`}, optional
+        Back-end to perform the actual Fourier transformation.
+
+    Returns
+    -------
+    gf_z : (..., Nz) complex np.ndarray
+        Laplace transformed Green's function for complex frequencies `z`.
+
+    See Also
+    --------
+    tt2z_trapz : Back-end: approximate integral by trapezoidal rule
+    tt2z_lin : Back-end: approximate integral by Filon's method
+
+    Raises
+    ------
+    ValueError
+        If neither the condition for retarded or advanced Green's function is
+        fulfilled.
+
+    Examples
+    --------
+    >>> tt = np.linspace(0, 150, num=1501)
+    >>> ww = np.linspace(-2, 2, num=501) + 1e-1j
+
+    >>> poles = 2*np.random.random(10) - 1  # partially filled
+    >>> weights = np.random.random(10)
+    >>> weights = weights/np.sum(weights)
+    >>> gf_ret_t = gt.pole_gf_ret_t(tt, poles=poles, weights=weights)
+    >>> gf_ft = gt.fourier.tt2z(tt, gf_ret_t, z=ww)
+    >>> gf_ww = gt.pole_gf_z(ww, poles=poles, weights=weights)
+
+    >>> import matplotlib.pyplot as plt
+    >>> __ = plt.plot(ww.real, gf_ww.imag, label='exact Im')
+    >>> __ = plt.plot(ww.real, gf_ft.imag, '--', label='DFT Im')
+    >>> __ = plt.plot(ww.real, gf_ww.real, label='exact Re')
+    >>> __ = plt.plot(ww.real, gf_ft.real, '--', label='DFT Re')
+    >>> __ = plt.legend()
+    >>> plt.show()
+
+    The function Laplace transform can be evaluated at abitrary contours,
+    e.g. for a semi-ceircle in the the upper half-plane.
+    Note, that close to the real axis the accuracy is bad, due to the
+    truncation at `max(tt)`
+
+    >>> z = np.exp(1j*np.pi*np.linspace(0, 1, num=51))
+    >>> gf_ft = gt.fourier.tt2z(tt, gf_ret_t, z=z)
+    >>> gf_z = gt.pole_gf_z(z, poles=poles, weights=weights)
+
+    >>> import matplotlib.pyplot as plt
+    >>> __ = plt.plot(z.real, gf_z.imag, '+', label='exact Im')
+    >>> __ = plt.plot(z.real, gf_ft.imag, 'x', label='DFT Im')
+    >>> __ = plt.plot(z.real, gf_z.real, '+', label='exact Re')
+    >>> __ = plt.plot(z.real, gf_ft.real, 'x', label='DFT Re')
+    >>> __ = plt.legend()
+    >>> plt.show()
+
+    Accuracy of the different back-ends:
+
+     * For large `z.imag`, `tt2z_lin` performs better.
+     * For intermediate `z.imag`, the quality depends on the relevant `z.real`.
+       For small `z.real`, the error of `tt2z_trapz` is more uniform;
+       for big `z.real`, `tt2z_lin` is a good approximation.
+     * For small `z.imag`, the methods are almost identical,
+       the truncation of `tt` dominates the error.
+
+    >>> import matplotlib.pyplot as plt
+    >>> for ii, eta in enumerate([1.0, 0.5, 0.1, 0.03]):
+    ...     ww.imag = eta
+    ...     gf_ww = gt.pole_gf_z(ww, poles=poles, weights=weights)
+    ...     gf_trapz = gt.fourier.tt2z(tt, gf_ret_t, z=ww, laplace=gt.fourier.tt2z_trapz)
+    ...     gf_lin = gt.fourier.tt2z(tt, gf_ret_t, z=ww, laplace=gt.fourier.tt2z_lin)
+    ...     __ = plt.plot(ww.real, abs((gf_ww - gf_trapz)/gf_ww),
+    ...                   label=f"z.imag={eta}", color=f"C{ii}")
+    ...     __ = plt.plot(ww.real, abs((gf_ww - gf_lin)/gf_ww), '--', color=f"C{ii}")
+    ...     __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.show()
+
+    """
+    retarded = np.all(tt >= 0) and np.all(z.imag >= 0)
+    advanced = np.all(tt <= 0) and np.all(z.imag <= 0)
+    if not (retarded or advanced):
+        raise ValueError("Laplace Transform only well defined if `tt>=0 and z.imag>=0`"
+                         " or `tt<=0 and z.imag<=0`")
+    if z.size == 0:  # consistent behavior for gufuncs
+        return np.empty_like(z)
+    return laplace(tt, gf_t, z)
+
+
+def _tau2polegf(gf_tau, beta, n_pole, moments=None, occ=False, weight=None) -> PoleGf:
+    tau = np.linspace(0, beta, num=gf_tau.shape[-1])
+    m1 = -gf_tau[..., -1] - gf_tau[..., 0]
+    if moments is None:  # = 1/z moment = jump of Gf at 0^{±}
+        moments = m1[..., newaxis]
+    else:
+        moments = np.asanyarray(moments)
+        if not np.allclose(m1, moments[..., 0]):
+            LOGGER.warning("Provided 1/z moment differs from jump."
+                           "\n mom: %s, jump: %s", moments[..., 0], m1)
+
+    def error_(width):
+        pole_gf = PoleGf.from_tau(gf_tau, n_pole=n_pole, beta=beta,
+                                  # if width is 0, no higher moments exist
+                                  moments=moments if width else m1[..., newaxis],
+                                  occ=occ, width=width, weight=weight)
+        gf_fit = pole_gf.eval_tau(tau, beta=beta)
+        return np.linalg.norm(gf_tau - gf_fit)
+
+    from scipy.optimize import minimize_scalar  # pylint: disable=import-outside-toplevel
+    opt = minimize_scalar(error_)
+    LOGGER.debug("Fitting error: %s Optimal pole-spread: %s", opt.fun, opt.x)
+    opt_pole_gf = PoleGf.from_tau(gf_tau, n_pole=n_pole, beta=beta, moments=moments,
+                                  occ=occ, width=opt.x, weight=weight)
+    return opt_pole_gf
+
+
+def tau2izp(gf_tau, beta, izp, moments=None, occ=False, weight=None):
+    r"""Fourier transform of the real Green's function `gf_tau` to `izp`.
+
+    Fourier transformation of a fermionic imaginary-time Green's function to
+    fermionic imaginary Padé frequencies `izp`.
+    We assume a real Green's function `gf_tau`, which is the case for
+    commutator Green's functions :math:`G_{AB}(τ) = ⟨A(τ)B⟩` with
+    :math:`A = B^†`. The Fourier transform `gf_iw` is then Hermitian.
+    If no explicit `moments` are given, this function removes
+    :math:`-G_{AB}(β) - G_{AB}(0) = ⟨[A,B]⟩`.
+
+    TODO: this function is not vectorized yet.
+
+    Parameters
+    ----------
+    gf_tau : (N_tau) float np.ndarray
+        The Green's function at imaginary times :math:`τ \in [0, β]`.
+    beta : float
+        The inverse temperature :math:`beta = 1/k_B T`.
+    izp : (N_izp) complex np.ndarray
+        Complex Padé frequencies at which the Fourier transform is evaluated.
+    moments : (m) float array_like, optional
+        High-frequency moments of `gf_iw`. If none are given, the first moment
+        is chosen to remove the discontinuity at :math:`τ=0^{±}`.
+
+    Returns
+    -------
+    gf_izp : (N_izp) complex np.ndarray
+        The Fourier transform of `gf_tau` for given Padé frequencies `izp`.
+
+    See Also
+    --------
+    tau2iw : Fourier transform to fermionic Matsubara frequencies.
+
+    pole_gf_from_tau : Function handling the fitting of `gf_tau`
+
+    Notes
+    -----
+    The algorithm performs in fact an analytic continuation instead of a
+    Fourier integral. It is however only evaluated on the imaginary axis, so
+    far the algorithm was observed to be stable
+
+    Examples
+    --------
+    >>> BETA = 50
+    >>> tau = np.linspace(0, BETA, num=2049, endpoint=True)
+    >>> izp, __ = gt.pade_frequencies(50, beta=BETA)
+
+    >>> poles = 2*np.random.random(10) - 1  # partially filled
+    >>> weights = np.random.random(10)
+    >>> weights = weights/np.sum(weights)
+    >>> gf_tau = gt.pole_gf_tau(tau, poles=poles, weights=weights, beta=BETA)
+    >>> gf_ft = gt.fourier.tau2izp(gf_tau, beta=BETA, izp=izp)
+    >>> gf_izp = gt.pole_gf_z(izp, poles=poles, weights=weights)
+
+    >>> import matplotlib.pyplot as plt
+    >>> __ = plt.plot(gf_izp.imag, label='exact Im')
+    >>> __ = plt.plot(gf_ft.imag, '--', label='FT Im')
+    >>> __ = plt.plot(gf_izp.real, label='exact Re')
+    >>> __ = plt.plot(gf_ft.real, '--', label='FT Re')
+    >>> __ = plt.legend()
+    >>> plt.show()
+
+    Results of `tau2izp` can be improved giving high-frequency moments.
+
+    >>> mom = np.sum(weights[:, np.newaxis] * poles[:, np.newaxis]**range(6), axis=0)
+    >>> for n in range(1, 6):
+    ...     gf = gt.fourier.tau2izp(gf_tau, izp=izp, moments=mom[:n], beta=BETA)
+    ...     __ = plt.plot(abs(gf_izp - gf), label=f'n_mom={n}', color=f'C{n}')
+    >>> __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.show()
+
+    The method is resistant against noise,
+    especially if there is knowledge of the noise:
+
+    >>> magnitude = 2e-7
+    >>> noise = np.random.normal(scale=magnitude, size=gf_tau.size)
+    >>> gf = gt.fourier.tau2izp(gf_tau + noise, izp=izp, moments=(1,), beta=BETA)
+    >>> __ = plt.plot(abs(gf_izp - gf), label='bare')
+    >>> gf = gt.fourier.tau2izp(gf_tau + noise, izp=izp, moments=(1,), beta=BETA,
+    ...                         weight=abs(noise)**-0.5)
+    >>> __ = plt.plot(abs(gf_izp - gf), label='weighted')
+    >>> __ = plt.axhline(magnitude, color='black')
+    >>> __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.tight_layout()
+    >>> plt.show()
+
+    >>> for n in range(1, 7, 2):
+    ...     gf = gt.fourier.tau2izp(gf_tau + noise, izp=izp, moments=mom[:n], beta=BETA)
+    ...     __ = plt.plot(abs(gf_izp - gf), '--', label=f'n_mom={n}', color=f'C{n}')
+    >>> __ = plt.axhline(magnitude, color='black')
+    >>> __ = plt.plot(abs(gf_izp - gf_ft), label='clean')
+    >>> __ = plt.legend()
+    >>> plt.yscale('log')
+    >>> plt.tight_layout()
+    >>> plt.show()
+
+    """
+    pole_gf = _tau2polegf(gf_tau, beta, n_pole=izp.size, moments=moments, occ=occ, weight=weight)
+    return pole_gf.eval_z(izp)
