@@ -16,14 +16,18 @@ Submodules
     honeycomb
     kagome
     sc
+    bcc
+    fcc
 
 """
-from . import (bethe, bethez, honeycomb, kagome, lieb, onedim, rectangular, sc,
-               square, triangular)
+from . import (bcc, bethe, bethez, fcc, honeycomb, kagome, lieb, onedim,
+               rectangular, sc, square, triangular)
 
 # silence warnings of unused imports
+assert bcc
 assert bethe
 assert bethez
+assert fcc
 assert honeycomb
 assert kagome
 assert lieb
