@@ -9,6 +9,8 @@ PATH = os.path.abspath(os.path.dirname(__file__))
 path.insert(0, os.path.join(PATH, os.pardir, os.pardir))
 
 import gftool
+import gftool.beb
+import gftool.cpa
 import gftool.matrix
 import gftool.fourier
 import gftool.pade as gt_pade
